@@ -193,7 +193,7 @@ static NV_STATUS uvm_user_channel_create(uvm_va_space_t *va_space,
     }
 
     if (user_channel->tsg.valid)
-        UVM_ASSERT(user_channel->tsg.max_subctx_count <= gpu->rm_info.maxSubctxCount);
+        UVM_ASSERT(user_channel->tsg.max_subctx_count <= gpu->max_subcontexts);
 
     // If num_resources == 0, as can happen with CE channels, we ignore base and
     // length.
@@ -968,6 +968,12 @@ NV_STATUS uvm8_test_check_channel_va_space(UVM_TEST_CHECK_CHANNEL_VA_SPACE_PARAM
     va_space = uvm_va_space_get(va_space_filp);
     uvm_va_space_down_read(va_space);
 
+    // We can do this query outside of the lock, but doing it within the lock
+    // simplifies error handling.
+    status = uvm_va_space_initialized(va_space);
+    if (status != NV_OK)
+        goto out;
+
     gpu = uvm_va_space_get_gpu_by_uuid(va_space, &params->gpu_uuid);
     if (!gpu || !uvm_processor_mask_test(&va_space->faultable_processors, gpu->id)) {
         status = NV_ERR_INVALID_DEVICE;
